"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyLocal = exports.stringifyRequest = exports.isRelativePath = exports.isAbsolutePath = void 0;
const path_1 = __importDefault(require("path"));
function isAbsolutePath(str) {
    return path_1.default.posix.isAbsolute(str) || path_1.default.win32.isAbsolute(str);
}
exports.isAbsolutePath = isAbsolutePath;
const RELATIVE_PATH_REGEXP = /^\.\.?[/\\]/;
function isRelativePath(str) {
    return RELATIVE_PATH_REGEXP.test(str);
}
exports.isRelativePath = isRelativePath;
function stringifyRequest(loaderContext, request) {
    if (typeof loaderContext.utils !== "undefined" &&
        typeof loaderContext.utils.contextify === "function") {
        return JSON.stringify(loaderContext.utils.contextify(loaderContext.context || loaderContext.rootContext, request));
    }
    const splitted = request.split("!");
    const { context } = loaderContext;
    return JSON.stringify(splitted
        .map(part => {
        // First, separate singlePath from query, because the query might contain paths again
        const splittedPart = part.match(/^(.*?)(\?.*)/);
        const query = splittedPart ? splittedPart[2] : "";
        let singlePath = splittedPart ? splittedPart[1] : part;
        if (isAbsolutePath(singlePath) && context) {
            singlePath = path_1.default.relative(context, singlePath);
            if (isAbsolutePath(singlePath)) {
                // If singlePath still matches an absolute path, singlePath was on a different drive than context.
                // In this case, we leave the path platform-specific without replacing any separators.
                // @see https://github.com/webpack/loader-utils/pull/14
                return singlePath + query;
            }
            if (isRelativePath(singlePath) === false) {
                // Ensure that the relative path starts at least with ./ otherwise it would be a request into the modules directory (like node_modules).
                singlePath = `./${singlePath}`;
            }
        }
        return singlePath.replace(/\\/g, "/") + query;
    })
        .join("!"));
}
exports.stringifyRequest = stringifyRequest;
function stringifyLocal(value) {
    return typeof value === "function" ? value.toString() : JSON.stringify(value);
}
exports.stringifyLocal = stringifyLocal;
