declare const rspackVersion: any, webpackVersion: any;
export { rspackVersion, webpackVersion as version };
export type { Asset, AssetInfo, Assets, CompilationParams, LogEntry } from "./Compilation";
export { Compilation } from "./Compilation";
export { Compiler } from "./Compiler";
export type { MultiCompilerOptions, MultiRspackOptions } from "./MultiCompiler";
export { MultiCompiler } from "./MultiCompiler";
import { RspackOptionsApply } from "./rspackOptionsApply";
export { RspackOptionsApply, RspackOptionsApply as WebpackOptionsApply };
export type { Chunk } from "./Chunk";
export type { ChunkGroup } from "./ChunkGroup";
export type { Module } from "./Module";
export { MultiStats } from "./MultiStats";
export { NormalModule } from "./NormalModule";
export type { NormalModuleFactory } from "./NormalModuleFactory";
export { RuntimeGlobals } from "./RuntimeGlobals";
export type { StatsAsset, StatsChunk, StatsCompilation, StatsError, StatsModule, StatsWarnings } from "./Stats";
export { Stats } from "./Stats";
import * as ModuleFilenameHelpers from "./lib/ModuleFilenameHelpers";
export { ModuleFilenameHelpers };
import Template = require("./Template");
export { Template };
export declare const WebpackError: ErrorConstructor;
export type { Watching } from "./Watching";
declare const sources: any;
export { sources };
import { applyRspackOptionsDefaults, getNormalizedRspackOptions } from "./config";
type Config = {
    getNormalizedRspackOptions: typeof getNormalizedRspackOptions;
    applyRspackOptionsDefaults: typeof applyRspackOptionsDefaults;
    getNormalizedWebpackOptions: typeof getNormalizedRspackOptions;
    applyWebpackOptionsDefaults: typeof applyRspackOptionsDefaults;
};
export declare const config: Config;
export type * from "./config";
export declare const util: {
    createHash: (algorithm: any) => any;
    cleverMerge: <T, O>(first: T, second: O) => T | O | (T & O);
};
export { default as EntryOptionPlugin } from "./lib/EntryOptionPlugin";
export { type OutputFileSystem } from "./util/fs";
export { cleanupGlobalTrace as experimental_cleanupGlobalTrace, registerGlobalTrace as experimental_registerGlobalTrace } from "@rspack/binding";
export type { BannerPluginArgument } from "./builtin-plugin";
export type { ProvidePluginOptions } from "./builtin-plugin";
export type { DefinePluginOptions } from "./builtin-plugin";
export type { ProgressPluginArgument } from "./builtin-plugin";
export type { EntryOptions } from "./builtin-plugin";
export { BannerPlugin } from "./builtin-plugin";
export { IgnorePlugin, type IgnorePluginOptions } from "./builtin-plugin";
export { ProvidePlugin } from "./builtin-plugin";
export { DefinePlugin } from "./builtin-plugin";
export { ProgressPlugin } from "./builtin-plugin";
export { EntryPlugin } from "./builtin-plugin";
export { DynamicEntryPlugin } from "./builtin-plugin";
export { ExternalsPlugin } from "./builtin-plugin";
export { HotModuleReplacementPlugin } from "./builtin-plugin";
export { EnvironmentPlugin } from "./lib/EnvironmentPlugin";
export { LoaderOptionsPlugin } from "./lib/LoaderOptionsPlugin";
export { LoaderTargetPlugin } from "./lib/LoaderTargetPlugin";
export { NormalModuleReplacementPlugin } from "./lib/NormalModuleReplacementPlugin";
import { NodeTargetPlugin } from "./builtin-plugin";
import NodeEnvironmentPlugin from "./node/NodeEnvironmentPlugin";
import NodeTemplatePlugin from "./node/NodeTemplatePlugin";
interface Node {
    NodeTargetPlugin: typeof NodeTargetPlugin;
    NodeTemplatePlugin: typeof NodeTemplatePlugin;
    NodeEnvironmentPlugin: typeof NodeEnvironmentPlugin;
}
export declare const node: Node;
import { ElectronTargetPlugin } from "./builtin-plugin";
interface Electron {
    ElectronTargetPlugin: typeof ElectronTargetPlugin;
}
export declare const electron: Electron;
import { EnableLibraryPlugin } from "./builtin-plugin";
interface Library {
    EnableLibraryPlugin: typeof EnableLibraryPlugin;
}
export declare const library: Library;
import { EnableWasmLoadingPlugin } from "./builtin-plugin";
interface Wasm {
    EnableWasmLoadingPlugin: typeof EnableWasmLoadingPlugin;
}
export declare const wasm: Wasm;
import { EnableChunkLoadingPlugin } from "./builtin-plugin";
interface JavaScript {
    EnableChunkLoadingPlugin: typeof EnableChunkLoadingPlugin;
}
export declare const javascript: JavaScript;
import { WebWorkerTemplatePlugin } from "./builtin-plugin";
interface Webworker {
    WebWorkerTemplatePlugin: typeof WebWorkerTemplatePlugin;
}
export declare const webworker: Webworker;
import { LimitChunkCountPlugin } from "./builtin-plugin";
import { RuntimeChunkPlugin } from "./builtin-plugin";
import { SplitChunksPlugin } from "./builtin-plugin";
interface Optimize {
    LimitChunkCountPlugin: typeof LimitChunkCountPlugin;
    RuntimeChunkPlugin: typeof RuntimeChunkPlugin;
    SplitChunksPlugin: typeof SplitChunksPlugin;
}
export declare const optimize: Optimize;
import { ModuleFederationPlugin } from "./container/ModuleFederationPlugin";
export type { ModuleFederationPluginOptions } from "./container/ModuleFederationPlugin";
import { ModuleFederationPluginV1 } from "./container/ModuleFederationPluginV1";
export type { ModuleFederationPluginV1Options } from "./container/ModuleFederationPluginV1";
import { ContainerPlugin } from "./container/ContainerPlugin";
import { ContainerReferencePlugin } from "./container/ContainerReferencePlugin";
export type { ContainerPluginOptions, Exposes, ExposesConfig, ExposesItem, ExposesItems, ExposesObject } from "./container/ContainerPlugin";
export type { ContainerReferencePluginOptions, Remotes, RemotesConfig, RemotesItem, RemotesItems, RemotesObject } from "./container/ContainerReferencePlugin";
export declare const container: {
    ContainerPlugin: typeof ContainerPlugin;
    ContainerReferencePlugin: typeof ContainerReferencePlugin;
    ModuleFederationPlugin: typeof ModuleFederationPlugin;
    ModuleFederationPluginV1: typeof ModuleFederationPluginV1;
};
import { ConsumeSharedPlugin } from "./sharing/ConsumeSharedPlugin";
import { ProvideSharedPlugin } from "./sharing/ProvideSharedPlugin";
import { SharePlugin } from "./sharing/SharePlugin";
export type { Consumes, ConsumesConfig, ConsumeSharedPluginOptions, ConsumesItem, ConsumesObject } from "./sharing/ConsumeSharedPlugin";
export type { Provides, ProvidesConfig, ProvideSharedPluginOptions, ProvidesItem, ProvidesObject } from "./sharing/ProvideSharedPlugin";
export type { Shared, SharedConfig, SharedItem, SharedObject, SharePluginOptions } from "./sharing/SharePlugin";
export declare const sharing: {
    ProvideSharedPlugin: typeof ProvideSharedPlugin;
    ConsumeSharedPlugin: typeof ConsumeSharedPlugin;
    SharePlugin: typeof SharePlugin;
};
export type { HtmlRspackPluginOptions } from "./builtin-plugin";
export type { SwcJsMinimizerRspackPluginOptions } from "./builtin-plugin";
export type { CopyRspackPluginOptions } from "./builtin-plugin";
export type { SourceMapDevToolPluginOptions } from "./builtin-plugin";
export type { EvalDevToolModulePluginOptions } from "./builtin-plugin";
export type { CssExtractRspackLoaderOptions, CssExtractRspackPluginOptions } from "./builtin-plugin";
export { HtmlRspackPlugin } from "./builtin-plugin";
export { SwcJsMinimizerRspackPlugin } from "./builtin-plugin";
export { SwcCssMinimizerRspackPlugin } from "./builtin-plugin";
export { CopyRspackPlugin } from "./builtin-plugin";
export { SourceMapDevToolPlugin } from "./builtin-plugin";
export { EvalSourceMapDevToolPlugin } from "./builtin-plugin";
export { EvalDevToolModulePlugin } from "./builtin-plugin";
export { CssExtractRspackPlugin } from "./builtin-plugin";
export type { SwcLoaderEnvConfig, SwcLoaderEsParserConfig, SwcLoaderJscConfig, SwcLoaderModuleConfig, SwcLoaderOptions, SwcLoaderParserConfig, SwcLoaderTransformConfig, SwcLoaderTsParserConfig } from "./builtin-loader/swc/index";
